
/**
*      TFC - UOC
       Aplicacion: Visor para senderismo con OpenLayers
       Autor: Alfredo Fernandez
       Fecha: 09/01/2012
*/


var map;
var rutasLayer, poisLayer, panoramioLayer;
var rutas = {};
var format = null;
var selectControl, popup;

var mobile = (/iphone|ipad|ipod|android|blackberry|mini|windows\sce|palm/i.test(navigator.userAgent.toLowerCase()));  

/**
 * Establece la altura mínima de #info_content
 */
var redimensionaInfoContent  = function(){
		
		if(mobile) document.getElementById('info_content').style.minHeight = ( windowSize().h - 32 ) + "px";
		else{
			document.getElementById('info_content').style.width = "80%";
			document.getElementById('info_content').style.maxHeight = ( windowSize().h - 100 ) + "px";
			document.getElementById('info_content').style.overflow = "auto";
			document.getElementById('info_content').style.margin = "auto";
		}
		
};
//Ponemos un listener que escuche las modificaciones de tamaño de #map.
document.getElementById('map').addEventListener("DOMSubtreeModified", redimensionaInfoContent, false);

//Iniciamos la web sin scrolls
document.getElementById("myBody").style.overflowX= "hidden";
document.getElementById("myBody").style.overflowY= "hidden";

//Los mobiles no soportan el atributo overflow: auto
if(!mobile) document.getElementById('info_content').style.overflow = "auto";

/**
 * Parseamos la respuesta del servidor en un desplegable
 */
var cargaRutasDesplegable = function(response) {
	eval("var data = " + response.responseText);
	var rutasEncuadre = new OpenLayers.Bounds(data['bbox']);
	map.setCenter(rutasEncuadre.getCenterLonLat(), 10);

	var select = document.getElementById('desplegableRutas');

	for ( var i = 0; i < data.features.length; i++) {
		var feature = data.features[i];
		select.options.add(new Option(feature.properties.nombre, feature.id));
		rutas[feature.id] = feature;
	}
};

/*
 * Cargamos en el mapa la ruta seleccionada en el desplegable 
 */
var cargaRuta = function() {
	cierraDescripcionRuta();
	
	if(popup)popup.destroy();
	
	var select = document.getElementById('desplegableRutas');
	if (undefined !== rutas[select.value]) {
		rutasLayer.removeAllFeatures();
		if (format == null)
			format = new OpenLayers.Format.GeoJSON();
		var features = format.read(rutas[select.value]);
		rutasLayer.addFeatures(features);
		map.zoomToExtent(rutasLayer.features[0].geometry.getBounds(), false);
		
		pidePois(rutasLayer.features[0].data.id);
		pidePanoramio();
		
	} else {
		alert("Tienes que escoger una ruta");
	}
};

/*
 * Mostramos la descripción de la ruta seleccionada en el desplegable 
 */
var mostrarDescripcionRuta = function(){
	
	var select = document.getElementById('desplegableRutas');
	if (undefined !== rutas[select.value]) {
		
		var ruta = rutas[select.value];
		format = new OpenLayers.Format.GeoJSON();
		var feature = format.read(ruta)[0];
		var content = "<div id=\"fitxa\">";
		content += "	<div>";
		content += "		<b>Ruta: " + feature.attributes.nombre + "</b><br>";
		content += "		<b>Dificultad:</b> " + feature.attributes.dificultad + "<br>";
		content += "		<b>Zona:</b> " + feature.attributes.zona + "<br>";
		content += "		<b>Descripción:</b> " + feature.attributes.descripcion + "<br>";
		content += "	</div>";
		if(feature.attributes.foto!="" && feature.attributes.foto!=null) content += "	<div class=\"foto\"><center><img src=\""+feature.attributes.foto+"\" alt=\"Imagen de la ruta\"></center></div>";
		content += "</div>";
		redimensionaInfoContent();
		if(mobile)document.getElementById("myBody").style.overflowY = "scroll";  
		document.getElementById('info_content').style.display = "block";
		document.getElementsByClassName('info_content_content')[0].innerHTML = content;


   
	} else {
		alert("Tienes que escoger una ruta");
	}
};


/*
 * Mostramos el perfil topográfico de la ruta seleccionada en el desplegable 
 */
var mostrarPerfilRuta = function(){
	
	var select = document.getElementById('desplegableRutas');
	if (undefined !== rutas[select.value]) {
		
		var ruta = rutas[select.value];
		format = new OpenLayers.Format.GeoJSON();
		var feature = format.read(ruta)[0];
		
		var content = "<div id=\"fitxa\"><div><h2>" + feature.attributes.nombre + "</h2>";
		content += "<div id=\"perfil\"></div></div>";

		redimensionaInfoContent();
		if(mobile)document.getElementById("myBody").style.overflowY = "scroll";  
		document.getElementById('info_content').style.display = "block";
		document.getElementsByClassName('info_content_content')[0].innerHTML = content;

		generaGrafico(feature);

   
	} else {
		alert("Tienes que escoger una ruta");
	}
};

/**
 * Cierra la ventana de descripción de una ruta.
 */
var cierraDescripcionRuta = function(){
	document.getElementById("myBody").style.overflowY = "hidden";  
	document.getElementById('info_content').style.display = "none";
	
};

/**
 * Pide los pois de una ruta al servidor
 */
var pidePois = function(ruta_id){
	var url = "http://geocodi.com/geoserver/wfs";
	new OpenLayers.Request.GET({
		url : url,
		params : {
					request:'GetFeature', typeName:'rutas:pois', CQL_FILTER:'ruta_id='+ruta_id,
					version: '1.0.0', outputFormat: 'json'
				 },
		success : cargaPois,
		failure : alert
	});
};

/**
 * Una vez recibidos los puntos los introduce en la capa poisLayer y le asigna un popup
 */
var cargaPois = function(response) {
    //Borramos los pois existentes
	poisLayer.removeAllFeatures();
	if (format == null) format = new OpenLayers.Format.GeoJSON();
	var features = format.read(response.responseText);
	//Establecemos el orden de visualización
	var orden = {1: 5, 2: 4, 3: 3, 4: 2, 5: 1};
	for(var i=0; i<features.length; i++){
		features[i].attributes.orden=orden[features[i].attributes.tipo_punto];
	}
	poisLayer.addFeatures(features);

};

/**
 * Abre el popup de un poi
 */
var muestraPopupPoi = function(feature) {
	var descripcion = feature.attributes.descripcion==null?"":feature.attributes.descripcion;
	var ruta = feature.attributes.ruta==null?"":feature.attributes.ruta;
	var content = "<div class=\"popupPoi\"><p>"+descripcion+"</p><p><b>"+ruta+"</b></p></div>";

	popup = new OpenLayers.Popup.FramedCloud("popupInformacion",
			new OpenLayers.LonLat(feature.geometry.x,feature.geometry.y),
			null, content, null, true, null);

	map.addPopup(popup, true);

};

/**
 * Pide Datos panoramio
 */
var pidePanoramio = function(ruta_id){

	var proj = new OpenLayers.Projection("EPSG:4326");
    var ext = map.getExtent().transform( map.getProjectionObject(), proj);
    var minx = ext.left;
    var miny = ext.bottom;
    var maxx = ext.right;
    var maxy = ext.top;
	
    var url = "http://www.panoramio.com/map/get_panoramas.php";

	new OpenLayers.Request.GET({
		url : url,
		params : {
					order: 'popularity',
					set: 'full',
					from: 0,
					to: 20,
					minx: minx,
					miny: miny,
					maxx: maxx,
					maxy: maxy,
					size:'thumbnail'
		   		},
		success : cargaPanoramio,
		failure : alert
	});
	
};

/**
 * Una vez recibidos los datos de Panoramio en la capa panoramioLayer y le asigna un popup
 */
var cargaPanoramio = function(response) {
    //Borramos los datos existentes
	panoramioLayer.removeAllFeatures();

	var json = new OpenLayers.Format.JSON();
	var panoramio = json.read(response.responseText);
    var features = [];
    
    for (var i = 0; i < panoramio.photos.length; i++)
    {
		upload_date = panoramio.photos[i].upload_date;
		owner_name = panoramio.photos[i].owner_name;
		photo_id = panoramio.photos[i].photo_id;
		longitude = panoramio.photos[i].longitude;
		latitude = panoramio.photos[i].latitude;
		pheight = panoramio.photos[i].height;
		pwidth = panoramio.photos[i].width;
		photo_title = panoramio.photos[i].photo_title;
		owner_url = panoramio.photos[i].owner_url;
		owner_id = panoramio.photos[i].owner_id;
		photo_file_url = panoramio.photos[i].photo_file_url;
		photo_url = panoramio.photos[i].photo_url;

        var punto = new OpenLayers.Geometry.Point(longitude,latitude);
		var proj = new OpenLayers.Projection("EPSG:4326");
		punto.transform(proj, map.getProjectionObject() );

		var atributos = {
			'upload_date' : upload_date,
			'owner_name':owner_name,
			'photo_id':photo_id,
			'longitude':longitude,
			'latitude':latitude,
			'pheight':pheight,
			'pwidth':pwidth,
			'pheight':pheight,
			'photo_title':photo_title,
			'owner_url':owner_url,
			'owner_id':owner_id,
			'photo_file_url':photo_file_url,	
			'photo_url':photo_url			 
		 };
			 
		features.push(new OpenLayers.Feature.Vector(punto,atributos));
    }
    panoramioLayer.addFeatures(features);
	
};


/**
 * Abre el popup de un dato de Panoramio
 */
var muestraPopupPanoramio = function(feature) {

	var content = "<div class=\"popupPoi\"><h2>"+ feature.attributes.photo_title + "</h2>";
	content += "<p>";
	content += "	<a href=\""+ feature.attributes.photo_url+"\">";
	content += "			<img src=\"http://mw2.google.com/mw-panoramio/photos/small/"+ feature.attributes.photo_id + ".jpg\" border=\"0\" alt=\"Ver Original\">";
	content += "	</a><br>";
	content += "Autor: <a href=\""+ feature.attributes.owner_url+"\">"+feature.attributes.owner_name +"</a>";
	content += "</p></div>";

    popup = new OpenLayers.Popup.FramedCloud("popupInformacion", 
    		new OpenLayers.LonLat(feature.geometry.x,feature.geometry.y),
            null,
            content,
            null, true, null);
    feature.popup = popup;
    map.addPopup(popup, true);

};



/**
 * Inicia el mapa
 */
var init = function() {
	
	OpenLayers.ProxyHost = "proxy.php?url=";

	OpenLayers.Lang.setCode('es');
	
	// Objeto mapa
	map = new OpenLayers.Map({
		div : "map", // Div de destino del mapa
		projection: new OpenLayers.Projection("EPSG:900913"), //Proyección del mapa
		displayProjection: new OpenLayers.Projection("EPSG:4326"), //Proyección en que se muestran las coordenadas
		controls : [ new OpenLayers.Control.Attribution(), // Control para añadir la descripción del mapa de fondo
		new OpenLayers.Control.TouchNavigation({ // Controles de navegación para móvil (touch)
			dragPanOptions : {
				enableKinetic : true
			}
		}), new OpenLayers.Control.ZoomPanel() // Botones de acercar/alejar
		]
	});


	// Capas de Google
	var gphy = new OpenLayers.Layer.Google("Google Physical", {
		type : google.maps.MapTypeId.TERRAIN
	});
	var gmap = new OpenLayers.Layer.Google("Google Streets", // the default
	{
		numZoomLevels : 20
	});
	var ghyb = new OpenLayers.Layer.Google("Google Hybrid", {
		type : google.maps.MapTypeId.HYBRID,
		numZoomLevels : 20
	});
	var gsat = new OpenLayers.Layer.Google("Google Satellite", {
		type : google.maps.MapTypeId.SATELLITE,
		numZoomLevels : 22
	});
	//Cargamos las imágenes
	map.addLayers([ gphy, gmap, ghyb, gsat ]);
	
	//Establecemos la capa por defecto
	map.setBaseLayer(gsat);

	//Control para gestionar las capas de google
	map.addControl(new OpenLayers.Control.LayerSwitcher());
	
	//Capa vectorial donde ponemos las rutas
	rutasLayer = new OpenLayers.Layer.Vector("Rutas", {
    	displayInLayerSwitcher: false,
        styleMap: new OpenLayers.StyleMap({
			strokeColor: "#FFCC33",
			strokeWidth: 4,
			strokeOpacity: 1
        })
    });
    
	map.addLayer(rutasLayer);

    //Capa de panoramio
    panoramioLayer = new OpenLayers.Layer.Vector("Panoramio fotos", {
    	displayInLayerSwitcher: true,
        styleMap: new OpenLayers.StyleMap({
            pointRadius: 7,
            fillOpacity: 1,
            externalGraphic: "iconos/panoramio-marker.png"
        })
    });
    map.addLayer(panoramioLayer);
    
	//Capa de marcadores donde ponemos los pois
    poisLayer = new OpenLayers.Layer.Vector("Puntos de Interés", {
    	displayInLayerSwitcher: true,
    	rendererOptions:{zIndexing: true},
        styleMap: new OpenLayers.StyleMap({
            externalGraphic: "iconos/${tipo_punto}.png",
            graphicZIndex: "${orden}",
            graphicOpacity: 1.0,
            graphicWith: 32,//16,
            graphicHeight: 37,//26,
            graphicYOffset: -30//-26
        })
    });

    map.addLayer(poisLayer);

    
    var selectControl = new OpenLayers.Control.SelectFeature([poisLayer, panoramioLayer], 
    		{
        		autoActivate:true
        	}
    );
    map.addControl(selectControl);
    
    
    poisLayer.events.on({
        "featureselected": function(e){muestraPopupPoi(e.feature);}
    });
    panoramioLayer.events.on({
        "featureselected": function(e){muestraPopupPanoramio(e.feature);}
    });
    
    //Cargamos las rutas en un desplegable
	var url = "http://geocodi.com/geoserver/wfs?SERVICE=WFS&VERSION=1.0.0&REQUEST=getfeature&TYPENAME=rutas:rutas&outputFormat=json";
	new OpenLayers.Request.GET({
		url : url,
		params : {},
		success : cargaRutasDesplegable,
		failure : alert
	});
	
	
	
	
	//Añadimos que muestre la coordenadas en caso de versión desktop
	if(!mobile){
	    map.addControl(new OpenLayers.Control.MousePosition());
	}
    
	//Centramos el mapa en el centro de Espana
	var centroEspana = new OpenLayers.LonLat(-2, 40);
	map.setCenter(centroEspana.transform(new OpenLayers.Projection("EPSG:4326"), map.getProjectionObject()), 6);

};

    
/**
 * Genera la imagen del perfil topográfico de una ruta 
 */
var generaGrafico = function(ruta) {

	var path = [];
	for(var i=0; i<ruta.geometry.components.length; i++){
		var punto = new OpenLayers.LonLat(ruta.geometry.components[i].x, ruta.geometry.components[i].y);
		punto.transform(map.getProjectionObject(), new OpenLayers.Projection("EPSG:4326"));
		path.push ( new google.maps.LatLng(punto.lat, punto.lon) );
	}
	
	var pathRequest = {
	        'path': path,
	        'samples': 256
		};
	//Instanciamos el servico de Google que nos permite obtener la altura de una coordenada.
	var elSvc = new google.maps.ElevationService();
	//Pedimos las alturas a al servicio de google
	elSvc.getElevationAlongPath(pathRequest, function(results, status){
		
		//Instanciamos el objeto para crear la gráfica.
		var chart = new google.visualization.ImageAreaChart(document.getElementById('perfil'));
	
		if (status == google.maps.ElevationStatus.OK) {
		    elevations = results;
		
		    var data = new google.visualization.DataTable();
		    data.addColumn('string', 'Ejemplo');
		    data.addColumn('number', 'Altura');
		    for (var i = 0; i < results.length; i++) {
		      data.addRow(['', elevations[i].elevation]);
		    }
		
		    var ancho = windowSize().w;
		    if(ancho > 480 ) ancho = 480;
		    ancho = ancho -40;

		    document.getElementById('perfil').style.display = 'block';
		    chart.draw(data, {
		      width: ancho,
		      height: 200,
		      title: 'Perfil topográfico'
		    });
		}
	});
};
 
/* Utilidades */
/*
 * Obtener las dimensiones de la ventana.
 */
function windowSize() {
	  var myWidth = 0, myHeight = 0;
	  if( typeof( window.innerWidth ) == 'number' ) {
	    //Non-IE
	    myWidth = window.innerWidth;
	    myHeight = window.innerHeight;
	  } else if( document.documentElement && ( document.documentElement.clientWidth || document.documentElement.clientHeight ) ) {
	    //IE 6+ in 'standards compliant mode'
	    myWidth = document.documentElement.clientWidth;
	    myHeight = document.documentElement.clientHeight;
	  } else if( document.body && ( document.body.clientWidth || document.body.clientHeight ) ) {
	    //IE 4 compatible
	    myWidth = document.body.clientWidth;
	    myHeight = document.body.clientHeight;
	  }
	  return {w:myWidth, h:myHeight};
}



